/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/kernel.h>

#include <zephyr/types.h>
#include <zephyr/kernel.h>

// USB drivers
#include <zephyr/usb/usb_device.h>
#include <zephyr/drivers/uart.h>

#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <soc.h>

#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/uuid.h>
#include <zephyr/bluetooth/gatt.h>
#include <zephyr/bluetooth/hci.h>

#include <bluetooth/services/nus.h>

#include <zephyr/settings/settings.h>

#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/sensor.h>

#include <stdio.h>

#include <zephyr/logging/log.h>

#define LOG_MODULE_NAME peripheral_uart
LOG_MODULE_REGISTER(LOG_MODULE_NAME);

BUILD_ASSERT(DT_NODE_HAS_COMPAT(DT_CHOSEN(zephyr_console), zephyr_cdc_acm_uart),
	     "Console device is not ACM CDC UART device");

static const struct gpio_dt_spec led0 = GPIO_DT_SPEC_GET(DT_ALIAS(led0), gpios);
static const struct gpio_dt_spec led1 = GPIO_DT_SPEC_GET(DT_ALIAS(led1), gpios);

void led_conf(){
	int ret1;
	int ret2;

	if (!device_is_ready(led0.port)) {
		return;
	}

	ret1 = gpio_pin_configure_dt(&led0, GPIO_OUTPUT_ACTIVE);

	ret2 = gpio_pin_configure_dt(&led1, GPIO_OUTPUT_ACTIVE);

	if (ret1 < 0) {
		return;
	}

	if (ret2 < 0) {
		return;
	}
}

/*USB funkcie*/
void enable_usb(){
	const struct device *const dev = DEVICE_DT_GET(DT_CHOSEN(zephyr_console));
	uint32_t dtr = 0;

	if (usb_enable(NULL)) {
		return;
	}

	/* Poll if the DTR flag was set */
	while (!dtr) {
		uart_line_ctrl_get(dev, UART_LINE_CTRL_DTR, &dtr);
		/* Give CPU resources to low priority threads. */
		k_sleep(K_MSEC(100));
	}
}
/*USB funkcie*/

/*Senzor init*/
static const struct device *temp_dev = DEVICE_DT_GET_ONE(sensirion_sht3xd);

void sensor_init(void)
{
	if (temp_dev == NULL || !device_is_ready(temp_dev)) {
		printk("no temperature device; using simulated data\n");
		temp_dev = NULL;
	} else {
		printk("temp device is %p, name is %s\n", temp_dev,
		       temp_dev->name);
	}
}
/*Senzor init*/

static struct bt_conn *current_conn;
static struct bt_conn *auth_conn;

#define DEVICE_NAME CONFIG_BT_DEVICE_NAME
#define DEVICE_NAME_LEN	(sizeof(DEVICE_NAME) - 1)

static const struct bt_data ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
	BT_DATA(BT_DATA_NAME_COMPLETE, DEVICE_NAME, DEVICE_NAME_LEN),
	BT_DATA_BYTES(BT_DATA_UUID128_ALL, BT_UUID_NUS_VAL)
};

static struct bt_le_ext_adv *adv;

static int create_advertising_coded(void)
{
	int err;
	struct bt_le_adv_param param =
		BT_LE_ADV_PARAM_INIT(BT_LE_ADV_OPT_CONNECTABLE |
				     BT_LE_ADV_OPT_EXT_ADV |
				     BT_LE_ADV_OPT_CODED,
				     BT_GAP_ADV_FAST_INT_MIN_2,
				     BT_GAP_ADV_FAST_INT_MAX_2,
				     NULL); // NULL mozne zmenit na direct advertising

	err = bt_le_ext_adv_create(&param, NULL, &adv);
	if (err) {
		printk("Failed to create advertiser set (err %d)\n", err);
		return err;
	}

	printk("Created adv: %p\n", adv);

	err = bt_le_ext_adv_set_data(adv, ad, ARRAY_SIZE(ad), NULL, 0);
	if (err) {
		printk("Failed to set advertising data (err %d)\n", err);
		return err;
	}

	return 0;
}


static void connected(struct bt_conn *conn, uint8_t err)
{
	char addr[BT_ADDR_LE_STR_LEN];
	struct bt_conn_info info;

	if (err) {
		LOG_ERR("Connection failed (err %u)", err);
		return;
	}

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));
	LOG_INF("Connected %s", addr);

	current_conn = bt_conn_ref(conn);

	gpio_pin_toggle_dt(&led0);

	err = bt_conn_get_info(conn, &info);
	if (err) {
		printk("Failed to get connection info (err %d)\n", err);
	} else {
		const struct bt_conn_le_phy_info *phy_info;
		phy_info = info.le.phy;

		printk("Connected: %s, tx_phy %u, rx_phy %u\n",
		       addr, phy_info->tx_phy, phy_info->rx_phy);
	}

}

static void disconnected(struct bt_conn *conn, uint8_t reason)
{
	char addr[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

	LOG_INF("Disconnected: %s (reason %u)", addr, reason);

	if (auth_conn) {
		bt_conn_unref(auth_conn);
		auth_conn = NULL;
	}

	if (current_conn) {
		bt_conn_unref(current_conn);
		current_conn = NULL;
		gpio_pin_toggle_dt(&led0);
	}

}

static void security_changed(struct bt_conn *conn, bt_security_t level,
			     enum bt_security_err err)
{
	char addr[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

	if (!err) {
		LOG_INF("Security changed: %s level %u", addr, level);
	} else {
		LOG_WRN("Security failed: %s level %u err %d", addr,
			level, err);
	}
}

BT_CONN_CB_DEFINE(conn_callbacks) = {
	.connected    = connected,
	.disconnected = disconnected,
	.security_changed = security_changed,
};

// spracovanie prijatych dat
static void bt_receive_cb(struct bt_conn *conn, const uint8_t *const data,
			  uint16_t len)
{
	int err;
	char addr[BT_ADDR_LE_STR_LEN] = {0};

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, ARRAY_SIZE(addr));

	LOG_INF("Received data from: %s", addr);

	int sign_temp = (int) ((uint8_t *)data)[0];

	printf("Prijate data %d\n",sign_temp);

}

static struct bt_nus_cb nus_cb = {
	.received = bt_receive_cb,
};

void send_senzor_data(void){
	struct sensor_value temp_value, hum_value; // struktura pre ulozenie dat senzora  

		uint8_t htm[5]; // ulozenie hodnoty teploty na poslanie // static uint8_t htm[5]; on posielal
		static double temperature, humidity;
	    int  base_temp, base_hum;
		int fractional_temp,fractional_hum;
		double int_part_temp, int_part_hum;
		int r;

		r = sensor_sample_fetch(temp_dev); // ziskanie sample zo senzora

		if (r) {
			printk("sensor_sample_fetch failed return: %d\n", r);
		}

		r = sensor_channel_get(temp_dev, SENSOR_CHAN_AMBIENT_TEMP,  // ziskanie hodnoty teploty
				       &temp_value);
		if (r) {
			printk("sensor_channel_get temperature failed return: %d\n", r);
		}

		r = sensor_channel_get(temp_dev, SENSOR_CHAN_HUMIDITY,  // ziskanie hodnoty vlhkosti
				       &hum_value);
		if (r) {
			printk("sensor_channel_get humidity failed return: %d\n", r);
		}

		temperature = sensor_value_to_double(&temp_value);
		if(temperature<0){
			htm[0] = (uint8_t) 1;
		}
		if(temperature>0){
			htm[0] = (uint8_t) 0;
		}

		int_part_temp = (double)((int)temperature); 
        fractional_temp = (int)((temperature - int_part_temp) * 100); 
        base_temp = (int)  int_part_temp;
		htm[1] = (uint8_t) base_temp;
		htm[2] = (uint8_t) fractional_temp;

		printf("temperature is %d.%d C\n", base_temp, fractional_temp);

		humidity = sensor_value_to_double(&hum_value);

		int_part_hum = (double)((int)humidity); 
        fractional_hum = (int)((humidity - int_part_hum) * 100); 
        base_hum = (int)  int_part_hum;
		htm[3] = (uint8_t) base_hum;
		htm[4] = (uint8_t) fractional_hum;

		printf("Humidity is %d.%d C\n", base_hum, fractional_hum);

		if (bt_nus_send(NULL, htm, sizeof(htm))) {  // zmenit co chceme poslat
			LOG_WRN("Failed to send data over BLE connection");
		}
}


void main(void)
{
	led_conf();
	enable_usb();

	printk("LOL\n");

	LOG_ERR("Getting handles from NUS service.");

	int err = 0;

		err = bt_enable(NULL);

		LOG_INF("Bluetooth initialized");

		err = settings_load();
		if (err) {
		LOG_ERR("Failed to setting load (err: %d)", err);
		return;
	}


		err = bt_nus_init(&nus_cb);
		if (err) {
		LOG_ERR("Failed to initialize UART service (err: %d)", err);
		return;
	}

	err = create_advertising_coded();
	if (err) {
		printk("Advertising failed to create (err %d)\n", err);
		return;
	}

	err = bt_le_ext_adv_start(adv, NULL);
	if (err) {
		LOG_ERR("Advertising failed to start (err %d)", err);
		return;
	}

	
	for(;;){
	if(current_conn!=NULL){
	k_sleep(K_SECONDS(2));
	send_senzor_data();
	printf("Poslane\n");
	}
	if(current_conn==NULL){
	// zacni advertising po odpojeni
	bt_le_ext_adv_start(adv, NULL);
	}
	}


	
}

